/***************************************************************************
    fifo.c  -  for FIFO handling
    -----------------------
    begin                : pi lut 11 09:59:54 CET 2000
    copyright            : (C) 2000 by Tomasz Fiszer
    email                : profis@kki.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
#include <syslog.h>
#include <fcntl.h>
#include "defs.h"
#include "utils.h"
#include "struct.h"


//
// FifoExchange
//
int FifoExchange(UPS_INFO_FIFO *Ups, int iNonBlock)
{
    int	fe;

    mkfifo(PS_FIFO_SERVER, 0777);
    if (iNonBlock == 1)
	fe = open(PS_FIFO_SERVER, O_NONBLOCK | O_WRONLY);
    if (iNonBlock == 0)
	fe = open(PS_FIFO_SERVER, O_WRONLY);
    
    if (fe == -1)
    {
	if (errno != ENXIO)
	{
	    close(fe);
	    unlink(PS_FIFO_SERVER);
	    return FAILURE;
	}
	close(fe);
	unlink(PS_FIFO_SERVER);
	return SUCCESS_ONESIDE;
    }
    if (Ups != NULL)
    {
	if (write(fe, Ups, sizeof(UPS_INFO_FIFO)) != sizeof(UPS_INFO_FIFO))
	{
	    close(fe);
	    unlink(PS_FIFO_SERVER);
	    return FAILURE;
	}
	else	
	{
	    close(fe);
	    return SUCCESS_TWOSIDE;
	}
    }
    close(fe);
    return SUCCESS;
}


//
// ReadFifo
//
int ReadFifo(UPS_INFO_FIFO *Ups, int iNonBlock)
{
    int		fe,
		iSize;
		
    if (mkfifo(PS_FIFO_SERVER, 0777) == -1)
    {
	if (errno != EEXIST)
	{
	    unlink(PS_FIFO_SERVER);
	    return FAILURE;
	}
    }
    if (iNonBlock == 1)
	fe = open(PS_FIFO_SERVER, O_NONBLOCK | O_SYNC | O_RDWR);
    if (iNonBlock == 0)
	fe = open(PS_FIFO_SERVER, O_SYNC | O_RDWR);
    
    if (fe == -1)
    {
	close(fe);
	unlink(PS_FIFO_SERVER);
	return FAILURE;
    }
    iSize = read(fe, Ups, sizeof(UPS_INFO_FIFO));
    if (iSize != sizeof(UPS_INFO_FIFO))
    {
        close(fe);
	unlink(PS_FIFO_SERVER);
        return FAILURE;
    }
    else
    {
        close(fe);
	unlink(PS_FIFO_SERVER);
	return SUCCESS_TWOSIDE;
    }
    close(fe);
    unlink(PS_FIFO_SERVER);
    return SUCCESS;
}

//
// End of file
//
