/************************************************************************
 *  pslock.c -- Lock file managing functions				*
 *  Copyright (C) 2000 Tomasz Fiszer <profis@friko7.onet.pl>		*
 *  All rights reserved.						*
 *  Portions Copyright by:						*
 *  	Brian Schau <bsc@fleggaard.dk>					*
 *      Facchetti Riccardo <fizban@tin.it>				*
 ************************************************************************/
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <netdb.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "defs.h"
#include "struct.h"
#include "utils.h"

int Finish(UPS_INFO *Upsinfo)
{
    close(Upsinfo->iLockFile);
    Upsinfo->iLockFile = -1;
    return (Upsinfo->iLockFile);
}

//
// Checking lock file
//
int CheckLockFile(UPS_INFO *Upsinfo)
{
    int		iError		= 0,
		iSize		= 0,
		iStalePID	= 0;
    char	pIDBuf[16];

    if (Upsinfo->szLockFilePath[0] == 0)
    {
	PrintToLog("Error: Path to Lock file not exist!\n");
        return _ERROR_LOCKFILE;
    }

    Upsinfo->iLockFile = open(Upsinfo->szLockFilePath, O_RDONLY);
    if (Upsinfo->iLockFile < 0) 
    {
	if (errno == ENOENT)
	    return _SUCCESS_LOCKFILE;
	else
	{
/*
	    PrintToLog("Error (%d): %s%s!\nDescription: %s\n",
		    errno, 
		    "Opening lock file!",
		    Upsinfo->szLockFilePath, 
		    strerror(errno));
*/
	    return _ERROR_LOCKFILE;
	}
    }
    
    iSize = read(Upsinfo->iLockFile, &pIDBuf, 15);
    if (iSize == -1) 
    {
/*
        PrintToLog("Error (%d): %s%s!\nDescription: %s\n",
		    errno, 
		    "Reading lock file!",
		    Upsinfo->szLockFilePath, 
		    strerror(errno));
*/
	iError = _ERROR_LOCKFILE;
	Finish(Upsinfo);
	return iError;
    }
/*
    else
        PrintToLog("errno (%d): read() %s%d\n",
		    errno, 
		    "iSize=",
		    iSize);
*/

    if (iSize == 0 || (sscanf(pIDBuf, "%d", &iStalePID) != 1)) 
    {
        if (errno)
	    PrintError("Error: In sscanf()!", errno, 1);
/*
        PrintToLog("Error (%d): %s%s!\nDescription: %s\n",
		    errno, 
		    "In sscanf!",
		    Upsinfo->szLockFilePath, 
		    strerror(errno));
*/
	iError = _ERROR_LOCKFILE;
	Finish(Upsinfo);
	return iError;
    }

    if (iStalePID == getpid()) 
    {
	iError = _ERROR_LOCKFILE_EXIST;
	Finish(Upsinfo);
	return iError;
    }
/*
    else
        PrintToLog("errno (%d): getpid(%d) iStalePID%d\n; Desc.: %s\n",
		    errno, 
		    getpid(),
		    iStalePID,
		    strerror(errno));
*/

    if ((kill(iStalePID, 0) == -1) && (errno == ESRCH))
    {
/*
        PrintToLog("errno (%d): getpid(%d) iStalePID%d\n; Desc.: %s\n",
		    errno, 
		    getpid(),
		    iStalePID,
		    strerror(errno));
*/
	unlink(Upsinfo->szLockFilePath);
	iError = _SUCCESS_LOCKFILE;
	Finish(Upsinfo);
	return iError;
    }
/*
    PrintToLog("Error (%d): %s%s!\nDescription: %s\n",
		errno, 
		"Finishing check of lock file!",
		Upsinfo->szLockFilePath, 
		strerror(errno));
*/
    return _ERROR_LOCKFILE;
}

//
// Creating lock file
//
int CreateLockFile(UPS_INFO *Upsinfo, int iKPw)
{
    char	pIDBuf[15];
    int		iError,
		iSize;
    
    iError = CheckLockFile(Upsinfo);

    switch (iError) 
    {
	case _ERROR_LOCKFILE_EXIST:
//	    PrintToLog("Error: Lock file exist!\n");
	    return _SUCCESS_LOCKFILE;
	case _ERROR_LOCKFILE:
//	    PrintToLog("Error: Unknown!\n");
	    return _ERROR_LOCKFILE;
	case _SUCCESS_LOCKFILE:
	    break;
    }
    Upsinfo->iLockFile = open(
	Upsinfo->szLockFilePath, 
	O_CREAT | O_EXCL | O_RDWR, 
	0644);
    if (Upsinfo->iLockFile < 0) 
    {
	PrintError("Error while creating lock file!", errno, 1);
	PrintToLog("Error: When creating Lock file!\n");
	if (iKPw)
	    Finish(Upsinfo);
	return _ERROR_LOCKFILE;
    }
    if (sprintf(pIDBuf, "%010d", getpid()) <= 0) 
    {
	PrintToLog("Error: Sprintf error!\n");
	iError = _ERROR_LOCKFILE;
	Finish(Upsinfo);
	return _ERROR_LOCKFILE;
    }
    
    iSize = write(Upsinfo->iLockFile, pIDBuf, strlen(pIDBuf));
    if (iSize != strlen(pIDBuf)) 
    {
	PrintToLog("Error: Writing to Lock file!\n");
	iError = _ERROR_LOCKFILE;
	Finish(Upsinfo);
	return _ERROR_LOCKFILE;
    }
    close(Upsinfo->iLockFile);
//    PrintToLog("Lock file creation success!\n");
    return _SUCCESS_LOCKFILE;
}

//
// deleting lock file
//
void DeleteLockFile(UPS_INFO *Upsinfo)
{
    int	iError;

/*    
    PrintToLog(
	"DeleteLockFile: Entering...\n");
*/
    if (Upsinfo->szLockFilePath[0] != 0) 
    {
        iError = CheckLockFile(Upsinfo);
        switch (iError) 
        {
	    case _ERROR_LOCKFILE_EXIST:
		break;
	    default:
/*
		PrintToLog(
		    "DeleteLockFile:\n"
		    "Cannot delete Lock file because it isn't our!\n");
*/
		return;
	}
	if (Upsinfo->iLockFile != -1) 
	{
	    PrintError(
		"DeleteLockFile:"
		"I will try to delete lock file after closing it, ok?\n",
		0, 0);
/*
	    PrintToLog(
	    	"DeleteLockFile:"
		"I will try to delete lock file after closing it, ok?\n");
*/
	    Finish(Upsinfo);
	}
	unlink(Upsinfo->szLockFilePath);
    }
/*
    PrintToLog(
	"DeleteLockFile: Leaving.\n");
*/
}
