/***************************************************************************
    struct.h  -  structures
    -------------------------------------
    Begin                : pia lut 11 09:59:54 CET 2000
    Copyright            : (C) 2000 by Tomasz Fiszer
    Email                : profis@kki.net.pl
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef _STRUCT_H_
#define _STRUCT_H_

#define MAX_PROBE_FIFO		3
#define MAX_STRING		256


/********************************************************************
****************** UPS Configuration and State **********************
********************** by Tomasz Fiszer *****************************
********************************************************************/
#ifndef _UPS_INFO_STRUCT_
#define _UPS_INFO_STRUCT_
// UPS_INFO
// This structure store all information 
// about UPS state and parameters.
//
typedef struct _UPS_INFO
{
    int			iLastError;
    //
    // Serial port configuration
    //
    unsigned char	szSerialDevice[4096];
    //
    // Serial port handle
    //
    int			iSerial;
    //
    // uiUpsType - contains ID number of UPS
    // 67 .. 82 - (NET-500 DPC .. AP-650 PRO)
    //
    unsigned int	uiUpsType;
    //
    // uiUpsState - contains info about UPS state (online,battery,standby)
    // 0	- Online; 
    // 1	- Battery;
    // 2	- Standby;
    // 3	- After UPS error;
    // 4>>	- Reserved;
    // 255	- Error;
    //
    unsigned long	ulUpsState;
    unsigned long	ulUpsStateCfg;
    //
    // uiUpsOperatingRanges - UPS state switching ranges
    // [0] - Upper	( 258V by default )
    // [1] - AVR	( 200V by default )
    // [2] - Lower	( 185V by default )
    //
    unsigned int	uiUpsOperatingRanges[3];
    //
    // uliUpsPowerValue - UPS power value (Watt)
    // This array contain (MAX_PROBE_FIFO) last power probes
    //
    unsigned long	ulUpsPowerValue[MAX_PROBE_FIFO];
    //
    // uliUpsAccuValue - UPS accumulator voltage value (Volt)
    // This array contain (MAX_PROBE_FIFO) last accu supply probes
    //
    unsigned long	ulUpsAccuValue[MAX_PROBE_FIFO];
    //
    // ulUpsACValue - UPS power supply in voltage value
    //
    unsigned long	ulUpsACValue[MAX_PROBE_FIFO];
    //
    // ulUpsACDifference - UPS power supply difference from last probe
    // in voltage value
    unsigned long	ulUpsACDifference;

    //
    // ulUpsConfig - UPS configuration
    // Bits:
    //	0	- Buzzer on/off
    //	1	- Timer on/off
    //	2	- Keyboard block (only NET-3000 at this time)
    //	3	- Set to Standby after shutdown
    //	4	- Battery failure or Old (isn't hard)
    //	5..7	- Reserved 
    //
    unsigned int	uiUpsConfig;
    // 
    // Lock file
    //
    unsigned char	szLockFilePath[MAX_STRING];
    int			iLockFile;
    //
    // Messages config
    //
    unsigned char	szMsgPowerFailure[MAX_STRING];
    unsigned char	szMsgPowerOK[MAX_STRING];
    unsigned char	szMsgStandby[MAX_STRING];
    unsigned char	szMsgTimeout[MAX_STRING];
    unsigned char	szMsgUPSError[MAX_STRING];
    unsigned char	szMsgShutdown[MAX_STRING];
    unsigned int	uiMsgEnabled;

    // Msg's Intervals
    unsigned int	uiMsgInterval;
    unsigned int	uiMsgInterval2;
    unsigned int	uiMsgShutdownInterval;

    // 
    // Commands config
    //
    unsigned char	szCmdPowerFailure[MAX_STRING];
    unsigned char	szCmdPowerOK[MAX_STRING];
    unsigned char	szCmdStandby[MAX_STRING];
    unsigned char	szCmdTimeout[MAX_STRING];
    unsigned char	szCmdUPSError[MAX_STRING];
    unsigned char	szCmdShutdown[MAX_STRING];
    unsigned int	uiCmdEnabled;
    // Cmd's Intervals
    unsigned int	uiCmdShutdownInterval;

    //
    // Program config
    //
    unsigned long	ulUserShutdownTimeout;
    
    //
    // Autotest interval and enablement
    //
    unsigned int	uiAutotestEnabled;
    unsigned long	ulAutotestInterval;
    //
    // System Variables
    //
    unsigned long	ulTimeToShutdown1;
    unsigned long	ulTimeToShutdown2;
    
    unsigned long	ulTimeToAutotest1;
    unsigned long	ulTimeToAutotest2;
    
    unsigned long	ulRecoveryTime;
    
    // Standby var's
    unsigned int	uiStandbyTimeout;
    unsigned long	ulOperation;
    unsigned int	uiNoLoginEnabled;
} UPS_INFO, *PUPS_INFO;
#endif /* _UPS_INFO_STRUCT+ */


#ifndef _UPS_INFO_FIFO_STRUCT_
#define _UPS_INFO_FIFO_STRUCT_
// UPS_INFO_FIFO
// This structure store all information 
// about UPS state and parameters for fifo.
//
typedef struct _UPS_INFO_FIFO
{
    int			iLastError;
    int			iSerial;
    unsigned int	uiUpsType;
    unsigned long	ulUpsState;
    unsigned long	ulUpsStateCfg;
    unsigned int	uiUpsOperatingRanges[3];

    unsigned long	ulUpsPowerValue[MAX_PROBE_FIFO];
    unsigned long	ulUpsAccuValue[MAX_PROBE_FIFO];
    unsigned long	ulUpsACValue[MAX_PROBE_FIFO];
    unsigned long	ulUpsACDifference;

    unsigned int	uiUpsConfig;
    int			iLockFile;
    unsigned int	uiMsgEnabled;
    unsigned int	uiMsgInterval;
    unsigned int	uiMsgInterval2;
    unsigned int	uiMsgShutdownInterval;
    unsigned int	uiCmdEnabled;
    unsigned int	uiCmdShutdownInterval;
    unsigned long	ulUserShutdownTimeout;
    unsigned int	uiAutotestEnabled;
    unsigned long	ulAutotestInterval;
    unsigned long	ulTimeToShutdown1;
    unsigned long	ulTimeToShutdown2;
    unsigned long	ulTimeToAutotest1;
    unsigned long	ulTimeToAutotest2;
    unsigned long	ulRecoveryTime;
    unsigned long	ulOperation;
    unsigned int	uiNoLoginEnabled;
} UPS_INFO_FIFO, *PUPS_INFO_FIFO;
#endif /*_UPS_INFO_FIFO_STRUCT_*/

#endif /* _STRUCT_H_ */

