/***************************************************************************
    ups.c  -  functions to handle communication options
    ------------------------
    begin	: pia lut 11 09:59:54 CET 2000
    copyright	: (C) 2000 by Tomasz Fiszer
    email	: profis@kki.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "ups.h"

#define VOLTAGEDIV1	372
#define VOLTAGEDIV2	352

static int 		_ThreadExit_	= 0;
static int		_comm_error	= 0;
static int		iFifoPVCounter	= 0;
static UPS_INFO_FIFO	UpsFifo;
unsigned long		ulTimeStart	= 0;
unsigned long		ulTimeNow	= 0;
unsigned long		ulAtstTimeStart	= 0;
unsigned long		ulAtstTimeNow	= 0;
unsigned long		ulRecoveryTime	= 0;
unsigned long		ulBuff		= 0;
unsigned long		ulMinute	= 0;


// Function to initialize frame of data
int Code(UPS_INFO *Ups)
{
TRACE;
    SendByte(Ups, 208);
    ReceiveByte(Ups, &cReceiveBuffer);
    switch(cReceiveBuffer)
    {
    case 207: return 0;
    case 208: return 1;
    default: _comm_error++; return -1;
    }
}

// Get all ups parameters (& work status)
int GetUpsParameters(UPS_INFO *Ups, int iType)
{
    int			iStBattery = 0, 
			iStStandby = 0;
    unsigned long	ulProbe;
    unsigned char	cRecBuffer[2];
    
    if (Ups->iSerial == -1)
	return EXIT_FAILURE;
    if (Ups->uiUpsType == 0)
    {
        if (Code(Ups) == 1)
	{
    	    SendByte(Ups, 173);
            ReceiveByte(Ups, (unsigned char *)&Ups->uiUpsType);
	    return SUCCESS;
	}
        else
	    return FAILURE;
    }
    // Line status
    if (Code(Ups))
    {
        SendByte(Ups, 175);
        ReceiveByte(Ups, &cReceiveBuffer);
	cReceiveBuffer &= 5;
	if (cReceiveBuffer == 4)
	{
	    iStBattery = TRUE;
	    iStStandby = FALSE;
	}
	else
	    if(cReceiveBuffer == 0) 
		iStBattery = FALSE;
    }
    else 
    {
        ReceiveByte(Ups, &cReceiveBuffer);
	if (Code(Ups)) 
	{
	    SendByte(Ups, 175);
	    ReceiveByte(Ups, &cReceiveBuffer);
	    cReceiveBuffer &= 5;
	    if(cReceiveBuffer == 4)	
	    {
		iStBattery = TRUE;
		iStStandby = FALSE;
	    }
	    else
		if(cReceiveBuffer == 0) 
		    iStBattery = FALSE;
	}
    }
    if (Code(Ups)) 
    {
	SendByte(Ups, 175);
        ReceiveByte(Ups, &cReceiveBuffer);
	cReceiveBuffer &= 1;
	if (cReceiveBuffer == 1) 
	{
	    iStStandby = TRUE;
	    iStBattery = FALSE;
	}
	else 
	    iStStandby = FALSE;
    }
    else
	return FAILURE;
    if (IsStatusSet(Ups->ulUpsState, ONLINE_WORK))
	ClearStatusWord(&Ups->ulUpsState, ONLINE_WORK);
    if (IsStatusSet(Ups->ulUpsState, BATTERY_WORK))
	ClearStatusWord(&Ups->ulUpsState, BATTERY_WORK);
    if (IsStatusSet(Ups->ulUpsState, STANDBY_WORK))
	ClearStatusWord(&Ups->ulUpsState, STANDBY_WORK);
    
    if (iStBattery == TRUE)
	SetStatusWord(&Ups->ulUpsState, BATTERY_WORK);
    else
	if (iStStandby) 
	    SetStatusWord(&Ups->ulUpsState, STANDBY_WORK);
	else
	    SetStatusWord(&Ups->ulUpsState, ONLINE_WORK);
    
    if (iType == 1)
    {
	// Accumulator voltage value (voltage)
	if (Code(Ups)) 
	{
	    SendByte(Ups, 189);
	    ReceiveByte(Ups, &cReceiveBuffer);
	    ulProbe = cReceiveBuffer;
	    ulProbe *= 150;
	    ulProbe /= 255;
	    // Fifo not yet implemented
	    Ups->ulUpsAccuValue[0] = (unsigned long) ulProbe;
	}
	if (Code(Ups))
	{
	    SendByte(Ups, 245);
	    ReceiveByte(Ups, &cRecBuffer[0]);
	    ReceiveByte(Ups, &cRecBuffer[1]);
	    if (Ups->uiUpsType > 72 && Ups->uiUpsType < 77)
		ulProbe = ((cRecBuffer[0] * 100) + (cRecBuffer[1] * 25600)) / VOLTAGEDIV2;
	    else
		ulProbe = ((cRecBuffer[0] * 100) + (cRecBuffer[1] * 25600)) / VOLTAGEDIV1;
	    Ups->ulUpsACDifference = (unsigned long)(Ups->ulUpsACValue[0] - ulProbe);
	    Ups->ulUpsACValue[iFifoPVCounter] = (unsigned long)ulProbe;
	    iFifoPVCounter++;
	    iFifoPVCounter %= 3;
	}
    }
    return SUCCESS;
}

// Get Ups name
char *GetUpsName(int iType)
{
    switch(iType)
    {
    case 67: return "NET 500 DPC";
    case 68: return "NET 700 DPC";
    case 69: return "NET 1000 DPC";
    case 70: return "NET 1400 DPC";
    case 71: return "NET 2200 DPC";
    // Aktualizacja typow ups-ow
    case 73: return "NET 700-DPC";
    case 74: return "NET 1000-DPC";
    case 75: return "NET 1400-DPC";
    case 76: return "NET 500-DPC";
    //
    case 81: return "AP 450 PRO";
    case 82: return "AP 650 PRO";
    default: return "Brak danych";
    }
}

// Check if message should be displayed
void CheckInfoMsg(UPS_INFO *Ups)
{
    unsigned long	ulTest = 0;

    ulTest = Ups->ulTimeToShutdown2 % Ups->uiMsgInterval;
    if (ulTest >= 0 && ulTest <= MAIN_LOOP_TIMER)
        BroadcastMessageInfo(Ups, Ups->szMsgTimeout);
}

// Check if Shutdown message should be displayed
void CheckShutdownMsg(UPS_INFO *Ups)
{
    if ((Ups->ulTimeToShutdown1 <= Ups->uiMsgShutdownInterval) &&
	!IsStatusSet(Ups->ulUpsStateCfg, MSG_SHUTDOWN_SEND))
    {
        SetStatusWord(&Ups->ulUpsStateCfg, MSG_SHUTDOWN_SEND);
        BroadcastMessage(Ups, Ups->szMsgShutdown);
    }
}

// Check if user has configured script to run before the system 
// is going to shutdown
void CheckUserShutdownCmd(UPS_INFO *Ups)
{
    if ((Ups->ulTimeToShutdown1 <= Ups->uiCmdShutdownInterval) &&
	!IsStatusSet(Ups->ulUpsStateCfg, CMD_SHUTDOWN_SEND))
    {
        SetStatusWord(&Ups->ulUpsStateCfg, CMD_SHUTDOWN_SEND);
        RunCommand(Ups, Ups->szCmdShutdown);
    }
}

// Execute shutdown procedure
int CheckShutdown(UPS_INFO *Ups)
{
    if (Ups->ulTimeToShutdown1 <= MAIN_LOOP_TIMER)
    {
	SwitchUPSToStandby(Ups);
	_ThreadExit_ = 1;
	PrintToLog("Execute system Shutdown procedure!\n");
	system(POWERSOFT_HALT);
	return 255;
    }
    return 0;
}

// Check if autotest need to be started
void CheckAutotest(UPS_INFO *Ups)
{
    if (Ups->uiAutotestEnabled)
    {
        time((time_t *)&ulAtstTimeNow);
        Ups->ulTimeToAutotest2 = ulAtstTimeNow - ulAtstTimeStart;
        if (Ups->ulTimeToAutotest2 > Ups->ulAutotestInterval)
	    Ups->ulTimeToAutotest2 = Ups->ulAutotestInterval;
        Ups->ulTimeToAutotest1 = Ups->ulAutotestInterval - Ups->ulTimeToAutotest2;
        if (Ups->ulTimeToAutotest1 <= MAIN_LOOP_TIMER)
	{
	    Ups->ulTimeToAutotest1 = Ups->ulAutotestInterval;
	    time((time_t *)&ulAtstTimeStart);
	    UPSAutotest(Ups);
        }
    }
}

// Switch to Battery mode
int SetBatteryWork(UPS_INFO *Ups)
{
    if(Code(Ups)) 
    {
	SendByte(Ups, 25);
	SendByte(Ups, 128);
	return TRUE;
    }
    else 
	return FALSE;
}

// Check AVR status
int IsAVRRunning(UPS_INFO *Ups)
{
    if(Code(Ups)) 
    {
	SendByte(Ups, 170);
	ReceiveByte(Ups, &cReceiveBuffer);
	if (cReceiveBuffer & 4) 
	    return TRUE;
	else 
	    return FALSE;
    }
    else
	return FAILURE;
}

// Latching System flags to enable sample caching
int LatchSysFlag(UPS_INFO *Ups)
{
    unsigned char ucSysFlag;
    
    if(Code(Ups))
    {
	SendByte(Ups, 141);
	ReceiveByte(Ups, (unsigned char *)&ucSysFlag);
	ucSysFlag &= 47;
	ucSysFlag += 192;
	if(Code(Ups))
	{
	    SendByte(Ups, 13);
	    SendByte(Ups, (unsigned char)ucSysFlag);
	    return SUCCESS;
	}
	else
	    return FAILURE;
    }
    else
	return FAILURE;
}

// Preparing autotest on Ups
int Autotest(UPS_INFO *Ups)
{
    unsigned char ucRec = 0;
    
    if (Code(Ups))
    {
	SendByte(Ups, 141);
	ReceiveByte(Ups, (unsigned char *)&ucRec);
	ucRec &= 237;
	ucRec += 2;
	if (Code(Ups))
	{
	    SendByte(Ups, 13);
	    SendByte(Ups, (unsigned char)ucRec);
	    return SUCCESS;
	} 
	else
	    return FAILURE;
    }
    else
	return FAILURE;
}

// Autotest
int UPSAutotest(UPS_INFO *Ups)
{
    const int	maxprobe	= 4;
    int	ak_op = 0, ak_om = 0, km_p = 0, km_m = 0, 
	iKmm[maxprobe], iKmmResult = 0,
	iLoopc = 0;
    memset(&iKmm, 0, sizeof(int) * maxprobe);
    if (!IsStatusSet(Ups->ulUpsState, BATTERY_WORK))
    {
	Autotest(Ups);
	if (LatchSysFlag(Ups) == SUCCESS)
	{
	    SetBatteryWork(Ups);
	    if (Code(Ups))
	    {
		SendByte(Ups, 196);
		ReceiveByte(Ups, (unsigned char *)&ak_op);
		ReceiveByte(Ups, (unsigned char *)&ak_om);
		ak_op = (ak_op * 1500) / 255;
		ak_om = (ak_om * 1500) / 255;
	    }
	    else
		return FAILURE;
	    
	    if ((ak_op + ak_om) > 1850)
	    {
		if (SetBatteryWork(Ups))
		{
		    usleep(250000);
		    if (!SetBatteryWork(Ups))
			return FAILURE;
		}
		else
		    return FAILURE;
	    }
	    else
		return SUCCESS; // Akumulator jest w stanie krytycznym
	}
    }
    usleep(100000);
    
    if (Ups->uiUpsType < 81)
    {
	SetBatteryWork(Ups);
PomiarStart:
	if (LatchSysFlag(Ups) == SUCCESS)
	{
	    if(Code(Ups))
	    {
		km_p = km_m = 0;
		SendByte(Ups, 232);
		ReceiveByte(Ups, (unsigned char *)&km_p);
		ReceiveByte(Ups, (unsigned char *)&km_m);
		km_p *= 100;
		km_m *= 100;
	    }
	    else
	        return FAILURE;
	}
	else
	    return FAILURE;
	if (iLoopc < 5)
	{
	    iKmm[iLoopc] = km_p + km_m;
	    if (iLoopc < 1)
		SetBatteryWork(Ups);
	    iLoopc++;
	    goto PomiarStart;
	}
	else
	{
	    iKmmResult = (iKmm[0] + iKmm[1] + iKmm[2] + iKmm[3]) / 4;
	}
    }
    
    Ups->ulUpsPowerValue[0] = GetTabLoad(Ups, iKmmResult);

    // Sleeping on 7 seconds because we don't want to see powerfail message
    usleep(700000);
    
    return SUCCESS;
}

// Calculating power value
int GetTabLoad(UPS_INFO *Ups, int iPwrVal)
{
    int	pwc, wx;
    int	iPowerVal[62] = 
	{
	    0, 30, 60, 90, 120, 150, 180, 210, 240, 270, 300, 330,
            360, 390, 420, 450, 480, 510, 540, 570, 600, 630, 660,
            690, 720, 750, 780, 810, 840, 870, 900, 930, 960, 990,
            1020, 1050, 1080, 1110, 1140, 1170, 1200, 1230, 1260,
            1290, 1320, 1350, 1380, 1410, 1440, 1470, 1500, 1530,
            1560, 1590, 1620, 1650, 1680, 1710, 1740, 1770, 1800
	};
    switch(Ups->uiUpsType)
    {
	case 67:	// NET-500
	case 76:
	{
	    iPwrVal = iPwrVal / 19;
	    pwc = 0;
	    for (wx = 100; wx < 280; wx += 18)
	    {
		if (wx >= iPwrVal)
		    break;
		pwc++;
	    }
	    return iPowerVal[pwc];
	}
	
	case 68:	// NET-700
	case 73:
	{
	    iPwrVal = iPwrVal / 23;
	    pwc = 0;
	    for (wx = 150; wx < 380; wx += 15)
	    {
		if (wx >= iPwrVal)
		    break;
		pwc++;
	    }
	    return iPowerVal[pwc];
	}
	
	case 69:	// NET-1000
	case 74:
	{
	    iPwrVal = iPwrVal / 23;
	    pwc = 0;
	    for (wx = 140; wx < 420; wx += 13)
	    {
		if (wx>=iPwrVal)
		    break;
		pwc++;
	    }
	    return iPowerVal[pwc];
	    break;
	}
	
	case 70:	// NET-1400
	case 75:
	{
	    iPwrVal = iPwrVal / 22;
	    pwc = 0;
	    for (wx = 140; wx < 460; wx += 9)
	    {
		if ( wx >= iPwrVal)
		    break;
		pwc++;
	    }
	    return iPowerVal[pwc];
	    break;
	}

	case 71:	// NET-2200
	{
	    int iPwrR, iPwrL, iMem, iDod;
   	    if(Code(Ups))
	    {
   		SendByte(Ups, 232);
      		ReceiveByte(Ups, (unsigned char *)&iPwrR);
      		ReceiveByte(Ups, (unsigned char *)&iPwrL);
	    }
	    else
	    {
		if(Code(Ups))
		{
		    SendByte(Ups, 232);
         	    ReceiveByte(Ups, (unsigned char *)&iPwrR);
         	    ReceiveByte(Ups, (unsigned char *)&iPwrL);
      		}
   	    }
   	    iPwrR *= 10;
	    iPwrR /= 15;
	    iPwrL *= 10;
	    iPwrL /= 15;
	    iMem = (iPwrR + iPwrL) / 2;
	    if (iMem > 5)
	    {
		iMem -= 5;
	    }
	    else
         	iMem -= iMem;
	    iMem *= 1190;
	    iMem /= 100;
	    iDod = iMem;
	    iDod /= 9;
	    iMem += iDod;
	    return iMem;
	}

	case 81:	// AP-450
	{
	    int iPwrSiec, iPwrAku, iDod;
	    if(Code(Ups))
	    {
		SendByte(Ups, 232);
		ReceiveByte(Ups, (unsigned char *)&iPwrSiec);
		ReceiveByte(Ups, (unsigned char *)&iPwrAku);
	    }
	    else
	    {
		if(Code(Ups))
		{
		    SendByte(Ups, 232);
		    ReceiveByte(Ups, (unsigned char *)&iPwrSiec);
		    ReceiveByte(Ups, (unsigned char *)&iPwrAku);
		}
	    }
	    if (!IsStatusSet(Ups->ulUpsState, BATTERY_WORK))
	    {
        	if (!IsAVRRunning(Ups))
		{
	      	    iPwrSiec *= 156;
		    iPwrSiec /= 80; //100;
		    //iPwrSiec /= 270;
		    return iPwrSiec;
		}
		else
		{
		    iPwrSiec *= 147;
		    iPwrSiec /= 80; //100;
		    //iPwrSiec /= 270;
		    return iPwrSiec;
		}
	    }
	    else
	    {
		iPwrAku *= 175;
		iPwrAku /= 100;
        	iDod = iPwrAku;
		iDod /= 50;
		iPwrAku += iDod;
		//iPwrAku *= 100;
		//iPwrAku /= 270;
		return iPwrAku;
	    }
	}

	case 82:	// AP-650
	{
      	    int iPwrSiec, iPwrAku, iDod;
	    if(Code(Ups))
	    {
		SendByte(Ups, 232);
		ReceiveByte(Ups, (unsigned char *)&iPwrSiec);
		ReceiveByte(Ups, (unsigned char *)&iPwrAku);
	    }
	    else
	    {
		if(Code(Ups))
		{
		    SendByte(Ups, 232);
		    ReceiveByte(Ups, (unsigned char *)&iPwrSiec);
		    ReceiveByte(Ups, (unsigned char *)&iPwrAku);
		}
	    }
	    if (!IsStatusSet(Ups->ulUpsState, BATTERY_WORK))
	    {
		if (!IsAVRRunning(Ups))
		{
		    iPwrSiec *= 248;
		    iPwrSiec /= 80; //100;
		    //iPwrSiec /= 420;
		    return iPwrSiec;
		}
		else
		{
		    iPwrSiec *= 238;
		    iPwrSiec /= 80; //100;
		    //iPwrSiec /= 420;
		    return iPwrSiec;
		}
	    }
	    else
	    {
		iPwrAku *= 220;
  	        iPwrAku /= 100;
		iDod = iPwrAku;
		iDod /= 10;
		iPwrAku += iDod;
		//iPwrSiec *= 100;
		//iPwrSiec /= 420;
		return iPwrAku;
	    }
	}
    }
    return 0;
}

// Switch UPS to standby mode
int SwitchUPSToStandby(UPS_INFO *Ups)
{
    double		dTimeSb;
    unsigned char	ucTime;

    if (Code(Ups))
    {
	dTimeSb = Ups->uiStandbyTimeout;
	dTimeSb /= 1.28;
	ucTime = (unsigned int) dTimeSb;
	SendByte(Ups, 28);
	SendByte(Ups, (unsigned char) ucTime);
	if (Code(Ups))
	{
	    SendByte(Ups, 13);
	    SendByte(Ups, 8);
	    return TRUE;
	}
    }
    return FALSE;
}

// Reset Ups standby timers
int SwitchUPSFromStandby(UPS_INFO *Ups)
{
    if (Code(Ups))
    {
        SendByte(Ups, 13);
        SendByte(Ups, 0);
        return TRUE;
    }
    return FALSE;
}

// Fill in UPS_INFO_FIFO struct for IPC
void FillFifoStruct(UPS_INFO_FIFO *UpsFifo, UPS_INFO Ups)
{
    UpsFifo->iLastError = Ups.iLastError;
    UpsFifo->iSerial = Ups.iSerial;
    UpsFifo->uiUpsType = Ups.uiUpsType;
    UpsFifo->ulUpsState = Ups.ulUpsState;
    UpsFifo->ulUpsStateCfg = Ups.ulUpsStateCfg;
    UpsFifo->uiUpsOperatingRanges[0] = Ups.uiUpsOperatingRanges[0];
    UpsFifo->uiUpsOperatingRanges[1] = Ups.uiUpsOperatingRanges[1];
    UpsFifo->uiUpsOperatingRanges[2] = Ups.uiUpsOperatingRanges[2];
    UpsFifo->uiUpsConfig = Ups.uiUpsConfig;
    UpsFifo->iLockFile = Ups.iLockFile;
    UpsFifo->uiMsgEnabled = Ups.uiMsgEnabled;
    UpsFifo->uiMsgInterval = Ups.uiMsgInterval;
    UpsFifo->uiMsgInterval2 = Ups.uiMsgInterval2;
    UpsFifo->uiMsgShutdownInterval = Ups.uiMsgShutdownInterval;
    UpsFifo->uiCmdEnabled = Ups.uiCmdEnabled;
    UpsFifo->uiCmdShutdownInterval = Ups.uiCmdShutdownInterval;
    UpsFifo->ulUserShutdownTimeout = Ups.ulUserShutdownTimeout;
    UpsFifo->ulTimeToShutdown1 = Ups.ulTimeToShutdown1;
    UpsFifo->ulTimeToShutdown2 = Ups.ulTimeToShutdown2;
    UpsFifo->ulTimeToAutotest1 = Ups.ulTimeToAutotest1;
    UpsFifo->ulTimeToAutotest2 = Ups.ulTimeToAutotest2;
    UpsFifo->ulRecoveryTime = Ups.ulRecoveryTime;
    
    UpsFifo->ulUpsPowerValue[0] = Ups.ulUpsPowerValue[0];
    UpsFifo->ulUpsAccuValue[0] = Ups.ulUpsAccuValue[0];
    UpsFifo->ulUpsACDifference = Ups.ulUpsACDifference;
    // Fifo with AC Value
    UpsFifo->ulUpsACValue[0] = Ups.ulUpsACValue[0];
    UpsFifo->ulUpsACValue[1] = Ups.ulUpsACValue[1];
    UpsFifo->ulUpsACValue[2] = Ups.ulUpsACValue[2];
}

// Stop thread
void StopThread()
{
    _ThreadExit_ = 1;
}

// Main loop
void MainUPSLoop(void *lpvArg)
{
    static UPS_INFO	Ups;
    unsigned char	cTest = 0;

    _ThreadExit_ = 0;
    
    memcpy(&Ups, lpvArg, sizeof(UPS_INFO));
    memset(&UpsFifo, 0, sizeof(UPS_INFO_FIFO));

    InitThreadSignals();
    SetStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_ONLINE);
    ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_BATTERY);
    ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_STANDBY);

    /* Inicjacja czasu powrotu do naladowania */
    ulRecoveryTime = 0;
    time((time_t *)&ulMinute);

    Ups.ulTimeToAutotest1 = Ups.ulAutotestInterval;
    time((time_t *)&ulAtstTimeStart);
    UPSAutotest(&Ups);    
    
    while(!_ThreadExit_)
    {
	if (Code(&Ups))
	{
	    SendByte(&Ups, 141);
	    ReceiveByte(&Ups, &cTest);
	}
	if (Code(&Ups))
	{
	    SendByte(&Ups, 156);
	    ReceiveByte(&Ups, &cTest);
	}
        if (GetUpsParameters(&Ups, 1) == EXIT_FAILURE)
	{
	    Terminate(0);
	    return;
	}
	FillFifoStruct(&UpsFifo, Ups);
	if (FifoExchange(&UpsFifo, 1) == SUCCESS_TWOSIDE)
	{
	    usleep(50000);
	    memset(&UpsFifo, 0, sizeof(UPS_INFO_FIFO));
	    if (ReadFifo(&UpsFifo, 0) == SUCCESS_TWOSIDE)
		if (UpsFifo.ulOperation == FIFO_AUTOTEST)
		    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK) == FALSE)
			UPSAutotest(&Ups);
	}
    
	if (IsStatusSet(Ups.ulUpsState, STANDBY_WORK) != 1)
	{
	    if (IsStatusSet(Ups.ulUpsState, BATTERY_WORK))
    	    {
		if (!IsStatusSet(Ups.ulUpsStateCfg, FIRST_TIME_BATTERY))
		{
		    BroadcastMessage(&Ups, Ups.szMsgPowerFailure);
		    RunCommand(&Ups, Ups.szCmdPowerFailure);
		    SetStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_BATTERY);
		    ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_ONLINE);
		    ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_STANDBY);
		    ClearStatusWord(&Ups.ulUpsStateCfg, MSG_SHUTDOWN_SEND);
		    ClearStatusWord(&Ups.ulUpsStateCfg, CMD_SHUTDOWN_SEND);
		    Ups.ulTimeToShutdown1 = Ups.ulUserShutdownTimeout - ulRecoveryTime;
		    Ups.ulTimeToShutdown2 = 0;
		    Ups.uiMsgInterval2 = Ups.uiMsgInterval;
		    time((time_t *)&ulTimeStart);
		    /* Czas powrotu do naladowania */
		    time((time_t *)&ulRecoveryTime);
		    UpsFifo.ulRecoveryTime = 0;
		    if (Ups.uiNoLoginEnabled == 1)
			CreateFile(NOLOGIN);
		    PrintToLog("%s\n", Ups.szMsgPowerFailure);
		}
		else
		{
		    /* zapisanie bufora dla RecoveryTime */
		    time((time_t *)&ulBuff);
		    time((time_t *)&ulTimeNow);
		    Ups.ulTimeToShutdown2 = ulTimeNow - ulTimeStart;
		    Ups.ulTimeToShutdown1 = Ups.ulUserShutdownTimeout;
		    Ups.ulTimeToShutdown1 -= (Ups.ulTimeToShutdown2);
		    // For info messages
		    CheckInfoMsg(&Ups);
		    // Check if it's Time to send msg or run cmd
		    CheckShutdownMsg(&Ups);
		    CheckUserShutdownCmd(&Ups);
		    if (CheckShutdown(&Ups) == 255)
		    {
			_ThreadExit_ = 1;
			break;
		    }
		}
	    }
	    else
	    {
		if (!IsStatusSet(Ups.ulUpsStateCfg, FIRST_TIME_ONLINE))
		{
		    /* Roznica czasu powrotu do naladowania */
		    time((time_t *)&ulMinute);
		    ulRecoveryTime = (ulBuff - ulRecoveryTime);
		    if (ulRecoveryTime > Ups.ulUserShutdownTimeout)
			ulRecoveryTime = Ups.ulUserShutdownTimeout;
		    Ups.ulTimeToShutdown1 = Ups.ulUserShutdownTimeout - ulRecoveryTime;
		    Ups.ulTimeToShutdown2 = 0;
		    BroadcastMessage(&Ups, Ups.szMsgPowerOK);
		    RunCommand(&Ups, Ups.szCmdPowerOK);
		    SetStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_ONLINE);
		    ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_BATTERY);
		    ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_STANDBY);
		    if (Ups.uiNoLoginEnabled == 1)
			DeleteFile(NOLOGIN);
		    PrintToLog("%s\n", Ups.szMsgPowerOK);
		}
		else
		{
		    /* Roznica czasu powrotu do naladowania */
		    time((time_t *)&ulBuff);
		    if ((ulBuff - ulMinute) >= RECOVERY_TIMEOUT)
		    {
			time((time_t *)&ulMinute);
			if (ulRecoveryTime >= Ups.ulRecoveryTime)
			    ulRecoveryTime -= Ups.ulRecoveryTime;
			else
			    ulRecoveryTime = 0;
			if (ulRecoveryTime > Ups.ulUserShutdownTimeout)
			    ulRecoveryTime = Ups.ulUserShutdownTimeout;
			Ups.ulTimeToShutdown1 = Ups.ulUserShutdownTimeout - ulRecoveryTime;
		    }
		    CheckAutotest(&Ups);
		}
	    }
	}
	else
	{
	    if (!IsStatusSet(Ups.ulUpsStateCfg, FIRST_TIME_STANDBY))
	    {
		BroadcastMessage(&Ups, Ups.szMsgStandby);
		RunCommand(&Ups, Ups.szCmdStandby);
		SetStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_STANDBY);
		ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_BATTERY);
		ClearStatusWord(&Ups.ulUpsStateCfg, FIRST_TIME_ONLINE);
		PrintToLog("%s\n", Ups.szMsgStandby);
	    }
	}
	if (_comm_error >= 4)
	{
	    if (_ThreadExit_ != 1)
	    {
		PrintError("Program utracil komunikacje z UPS-em!\n", 0, 0);
    	        printf("Zaczekaj na zakonczenie programu...\n");
    		PrintToLog("Program stracil komunikacje z UPS-em!\n");
		_ThreadExit_ = 1;
	    }
	}
	// Wait MAIN_LOOP_TIMER seconds
	Standby(MAIN_LOOP_TIMER);
    }
}
